﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.ComponentModel;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;

namespace XamDataGridSearchEngine
{
	/// <summary>
	/// Interaction logic for UserControl1.xaml
	/// </summary>
	public partial class SearchBar : UserControl
	{
        private XamDataGrid grid;
        private List<Record> resultRecords;
        private List<Cell> cells;
        private int currentIndex;
        public bool isOpen = false;

		public SearchBar()
		{
			this.InitializeComponent();
		}

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (isOpen)
            {
                Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                BeginStoryboard(closeBoard);

                searchBox.Text = "";

                isOpen = false;
            }
            else
            {
                Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                BeginStoryboard(openBoard);
                
                isOpen = true;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            grid = Utilities.GetAncestorFromType(sender as DependencyObject, typeof(XamDataGrid), false) as XamDataGrid;
            grid.KeyDown += UserControl_KeyDown;
            if (grid != null)
            {
                Binding tagBinding = new Binding();
                tagBinding.Source = searchBox;
                tagBinding.Path = new PropertyPath("Text");
                tagBinding.Mode = BindingMode.TwoWay;

                grid.SetBinding(XamDataGrid.TagProperty, tagBinding);
            }

            resultRecords = new List<Record>();
            cells = new List<Cell>();
        }

        private void searchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            string searchStr = (sender as TextBox).Text.ToLower();

            foreach (Record rec in resultRecords)
            {
                rec.Tag = "";
            }

            foreach (DataRecord record in grid.SelectedItems.Records)
            {
                foreach (Cell cell in record.Cells)
                {
                        CellValuePresenter.FromCell(cell).Background = Brushes.Transparent;
                }
            }

            resultRecords.Clear();
            
            if (searchStr.Equals(""))
            {
                foreach (DataRecord record in grid.SelectedItems.Records)
                {
                    foreach (Cell cell in record.Cells)
                    {
                            CellValuePresenter.FromCell(cell).Background = Brushes.Transparent;
                    }
                }
                return; 
            }
            

        //    if (grid != null && grid.Records is DataRecordCollection)
            if (grid != null && grid.SelectedItems.Records is SelectedRecordCollection)
            {
                foreach (DataRecord record in grid.SelectedItems.Records)
                {
                    foreach (Cell cell in record.Cells)
                    {
                        string cellString = cell.Value.ToString().ToLower();

                        if (cellString.Contains(searchStr))
                        {
                            //set the background color for cell
                            CellValuePresenter.FromCell(cell).Background = Brushes.Orange;
                           
                            record.Tag = "match";
                            resultRecords.Add(record);
                            break;
                        }
                    }
                }
            }

            if (grid.Records is GroupByRecordCollection)
            {
                foreach (GroupByRecord gr in grid.Records)
                {
                    AddMatchingRecords(gr, resultRecords, searchStr);
                }
            }

            if (resultRecords.Count > 0)
            {
                currentIndex = 0;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
            }
        }

        public void AddMatchingRecords(GroupByRecord gr, List<Record> recList, string text)
        {
            if (gr.HasChildren)
            {

                foreach (Record record in gr.ChildRecords)
                {
                    if (record is DataRecord)
                    {
                        foreach (Cell cell in (record as DataRecord).Cells)
                        {
                            string cellValue = cell.Value.ToString().ToLower();
                            if (cellValue.Contains(text))
                            {
                                
                                record.Tag = "match";
                                recList.Add(record);
                            }
                        }
                    }

                    if (record is GroupByRecord)
                    {
                        AddMatchingRecords(record as GroupByRecord, recList, text);
                    }

                }
                
            }
        }

        private void next_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex < resultRecords.Count - 1)
            {
                currentIndex++;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
                grid.ActiveRecord = resultRecords[currentIndex];
            }
            else
            {
                MessageBox.Show("No More Results Found");
            }
        }

        private void prev_Click(object sender, RoutedEventArgs e)
        {
            if (currentIndex > 0)
            {
                currentIndex--;
                grid.BringRecordIntoView(resultRecords[currentIndex]);
                grid.ActiveRecord = resultRecords[currentIndex];
            }
            else
            {
                MessageBox.Show("No More Results Found");
            }
        }

        private void UserControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.F &&  Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                
                if (isOpen)
                {
                    Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                    BeginStoryboard(closeBoard);
                    
                    searchBox.Text = "";

                    isOpen = false;
                }
                else
                {
                    Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                    BeginStoryboard(openBoard);

                    isOpen = true;
                }
            }
        }

        private void UserControl_KeyDown_1(object sender, KeyEventArgs e)
        {

        }


        //Clear the visual state of XamDataGrid 
        private void searchBox_LostFocus(object sender, RoutedEventArgs e)
        {

            foreach (DataRecord record in grid.SelectedItems.Records)
            {
                foreach (Cell cell in record.Cells)
                {
                   CellValuePresenter.FromCell(cell).Background = Brushes.Transparent;
                }
            }
        }


      

    }
}